/*
 * Binary tree database allocation and deallocation functions.
 *
 * Copyright 2003 Andrew Wood, distributed under the Artistic License.
 */

#include "config.h"
#include "database.h"

#ifdef USING_INTERNAL_DB

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include "btreei.h"


/*
 * Open the given database in the given way (new database, read-only, or
 * read-write); return a qdb_t or NULL on error.
 */
qdb_t qdb_open(char *file, qdb_open_t method)
{
	qdb_t db;
	FILE *fptr = NULL;
#ifdef HAVE_FCNTL
	int locktype = F_RDLCK;
#endif

	switch (method) {
	case QDB_NEW:
		fptr = fopen(file, "w+b");
		if (fptr == NULL)
			bt_lasterror = strerror(errno);
#ifdef HAVE_FCNTL
		locktype = F_WRLCK;
#endif
		break;
	case QDB_READONLY:
		fptr = fopen(file, "rb");
		if (fptr == NULL)
			bt_lasterror = strerror(errno);
		break;
	case QDB_READWRITE:
		fptr = fopen(file, "r+b");
		if (fptr == NULL)
			fptr = fopen(file, "w+b");
		if (fptr == NULL)
			bt_lasterror = strerror(errno);
#ifdef HAVE_FCNTL
		locktype = F_WRLCK;
#endif
		break;
	default:
		break;
	}

	if (fptr == NULL)
		return NULL;

	db = calloc(1, sizeof(*db));
	if (db == NULL) {
		bt_lasterror = strerror(errno);
		fclose(fptr);
		return NULL;
	}

	fseek(fptr, 0, SEEK_END);
	db->size = ftell(fptr);

	db->fptr = fptr;

#ifdef HAVE_FCNTL
	db->locktype = locktype;

	if (dbbt_lock(db, locktype)) {
		fclose(fptr);
		free(db);
		return NULL;
	}
#endif

	return db;
}


/*
 * Close the given database.
 */
void qdb_close(qdb_t db)
{
	if (db == NULL)
		return;

#ifdef HAVE_FCNTL
	while (db->lockcount > 0)
		dbbt_lock(db, F_UNLCK);
#endif

	fclose(db->fptr);

	free(db);
}

#endif				/* USING_INTERNAL_DB */

/* EOF */
